#include <fcntl.h>
#include <stdio.h>
#include <linux/i2c-dev.h>
#include "audio.h"
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <termios.h>

#define ZYBO_MAP_SIZE 0x1000
#define I2S_DATA_RX_L_REG 0x00 
#define I2S_DATA_RX_R_REG 0x04 
#define I2S_DATA_TX_L_REG 0x08 
#define I2S_DATA_TX_R_REG 0x0c 
#define I2S_STATUS_REG 0x10 

#define I2C_SLAVE 0x0703

int Fdiic;

int kbhit(void)
{
	struct termios oldt, newt;
	int ch;
	int oldf;

	tcgetattr(STDIN_FILENO, &oldt);
	newt = oldt;
	newt.c_lflag &= ~(ICANON | ECHO);
	tcsetattr(STDIN_FILENO, TCSANOW, &newt);
	oldf = fcntl(STDIN_FILENO, F_GETFL, 0);
	fcntl(STDIN_FILENO, F_SETFL, oldf | O_NONBLOCK);

	ch = getchar();

	tcsetattr(STDIN_FILENO, TCSANOW, &oldt);
	fcntl(STDIN_FILENO, F_SETFL, oldf);
	if(ch != EOF)
	{
		ungetc(ch, stdin);
		return 1;
	}
	return 0;
}

/******************************************************************************
 * Function to write 9-bits to one of the registers from the audio
 * controller.
 * @param	u8RegAddr is the register address.
 * @param	u16Data is the data word to write ( only least significant 9 bits).
  * @return	none.
 *****************************************************************************/
void AudioWriteToReg(unsigned char u8RegAddr, short u16Data) {

	unsigned char u8TxData[2], u8RxData[2];
	int BytesWritten, BytesRead;

	u8TxData[0] = (u8RegAddr << 1 ) | ((u16Data >> 8) & 0x01); // append msb of 9-bit data to the reg addr after shifting left
	u8TxData[1] = u16Data & 0xFF;

	BytesWritten = write(Fdiic, u8TxData, 2);
	if (BytesWritten != 2)
		printf("Error in writing %x at regaddr %x\n",u16Data, u8RegAddr);
}

/******************************************************************************
 * Configures Line-In input, ADC's, DAC's, Line-Out and HP-Out.
 * @param   none.
 * @return	none.
 *****************************************************************************/
void LineinLineoutConfig() {

	int i;
	// software reset
	AudioWriteToReg(R15_SOFTWARE_RESET, 0x000);
	// power mgmt: 0_00110010=>0,Power up, power up, OSC dn, out off, DAC up, ADC up, MIC off, LineIn up
	AudioWriteToReg(R6_POWER_MANAGEMENT, 0x030);
	// left ADC Input: 0_01010111=>0,mute disable, Line volume 0 dB
	AudioWriteToReg(R0_LEFT_ADC_INPUT,0x017);
	// right ADC Input: 0_00010111=>0,mute disable, Line volume 0 dB
	AudioWriteToReg(R1_RIGHT_ADC_INPUT,0x017);
	AudioWriteToReg(R2_LEFT_DAC_VOLUME,0x079);
	AudioWriteToReg(R3_RIGHT_DAC_VOLUME,0x079);
	// analog audio path: 0_00010010=>0,-6 dB side attenuation, sidetone off, DAC selected, bypass disabled, line input, mic mute disabled, 0 dB mic
	AudioWriteToReg(R4_ANALOG_AUDIO_PATH, 0x012);
	// digital audio path: 0_00000000=>0_000, clear offset, no mute, no de-emphasize, adc high-pass filter enabled
	AudioWriteToReg(R5_DIGITAL_AUDIO_PATH, 0x000);
	// digital audio interface: 0_00001110=>0, BCLK not inverted, slave mode, no l-r swap, normal LRC and PBRC, 24-bit, I2S mode
	AudioWriteToReg(R7_DIGITAL_AUDIO_INTERFACE, 0x00A);
	// Digital core:0_00000001=>0_0000000, activate core
	AudioWriteToReg(R9_ACTIVE, 0x001);
	// power mgmt: 0_00100010 0_Power up, power up, OSC dn, out ON, DAC up, ADC up, MIC off, LineIn up
	AudioWriteToReg(R6_POWER_MANAGEMENT, 0x022); // power mgmt: 001100010 turn on OUT

}


void bypass_input(void *zybo_ptr, void *gpio_ptr)
{
	unsigned u32DataL, u32DataR;

	while (1)
	{
		while(1) { //wait for RX data to become available
			if(*((unsigned *)(zybo_ptr + I2S_STATUS_REG)))	
				break;
			else 
				continue;
		}	
		*((unsigned *)(zybo_ptr + I2S_STATUS_REG)) = 0x1; //Clear data rdy bit
		u32DataL = *((unsigned *)(zybo_ptr + I2S_DATA_RX_L_REG));
		u32DataR = *((unsigned *)(zybo_ptr + I2S_DATA_RX_R_REG));

		if((*((unsigned *)(gpio_ptr + 8)))&0x01) { // through mode
			*((unsigned *)(zybo_ptr + I2S_DATA_TX_L_REG)) = u32DataL;
			*((unsigned *)(zybo_ptr + I2S_DATA_TX_R_REG)) = u32DataR;
		}
		else // mute mode
		{
			*((unsigned *)(zybo_ptr + I2S_DATA_TX_L_REG)) = 0;
			*((unsigned *)(zybo_ptr + I2S_DATA_TX_R_REG)) = 0;
		}
		if(kbhit()) {
			printf(" Done. Exiting...\n"); 
			break; 
		}
	}
}


int main(int argc, char *argv[])
{
	
	/* for Audio Control */
	int zybo_fd, Status;
	char *zybo_uiobf="/dev/uio3";
	void *zybo_ptr;

	/* for GPIO Control */
	int gpio_fd, sel;
	char *gpio_uiobf="/dev/uio0";
	void *gpio_ptr;

	Fdiic = open("/dev/i2c-0", O_RDWR); 
	if(Fdiic < 0) {
		printf("\nError initializing IIC");
		return -1;
	}

	Status = ioctl(Fdiic, I2C_SLAVE, IIC_SLAVE_ADDR); 
	if(Status < 0) {
		printf("\n Unable to set the address\n");
		return -1;
	}
	//Configure the Line in and Line out ports.
	//Call LineInLineOutConfig() for a configuration that
	//enables the HP jack too.
	LineinLineoutConfig();

	printf("Audio CODEC(SSM2603) initialized\n");

	/* audio core */
	zybo_fd = open(zybo_uiobf, O_RDWR);
	if (zybo_fd < 1) {
		printf("Invalid UIO device file:%s.\n", zybo_uiobf);
		return -1;
	}

	/* mmap the UIO devices */
	zybo_ptr = mmap(NULL, ZYBO_MAP_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED, zybo_fd, 0);

	/* GPIO core */
	gpio_fd = open(gpio_uiobf, O_RDWR);
	if (gpio_fd < 1) {
		printf("Invalid UIO device file:%s.\n", gpio_uiobf);
		return -1;
	}

	/* mmap the UIO devices */
	gpio_ptr = mmap(NULL, ZYBO_MAP_SIZE, PROT_READ|PROT_WRITE, MAP_SHARED, gpio_fd, 0);

	/* set directions, channel 0 => output, channel 1 => input */
	*((unsigned *)(gpio_ptr + 4)) = 0x0;	// output direction
	*((unsigned *)(gpio_ptr + 0)) = 0x1;	// output data

	/* set SW0 input direction*/
	*((unsigned *)(gpio_ptr + 0xC)) = 0x1;	// input direction

	printf("Put SW0 to UP position for Through mode, DN position to Mute mode\n");
	printf("Press any key to terminate the program\n");

	bypass_input(zybo_ptr, gpio_ptr); 

    return 0;

}


